import { Account, Caller, Chain, ContractBuilder, Receipt, TxResult, Wallet } from "./types";
export type CPChainMessagePayload = {
    status: "not_unlocked" | "allowed" | "rejected";
    address?: string;
    tx_hash?: string;
    chain?: Chain;
};
export type CPChainMessage = {
    type: "FROM_CPCHAIN";
    action: "connect" | "submit-tx" | "add-token";
    payload: CPChainMessagePayload;
};
export type SubmitTxRepl = {
    status: 'allowed' | 'rejected';
    tx_hash?: string;
};
export declare function submitTx(payload: {
    to: string;
    data: string;
    value: number;
}): Promise<SubmitTxRepl>;
export declare function extensionWalletExists(): Promise<boolean>;
export declare function connectWallet(): Promise<CPChainMessagePayload>;
export declare class ExtensionTxResult implements TxResult {
    hash: string;
    constructor(hash: string);
    wait(): Promise<Receipt>;
}
export declare class ExtensionCaller implements Caller {
    private method;
    private to;
    private data;
    _value: number;
    _gas: number;
    constructor(method: ABIMethod, to: string, data: string);
    view(): Promise<unknown>;
    gasLimit(gas: number): Caller;
    value(value: number): Caller;
    submit(): Promise<TxResult>;
}
export type ABIMethod = {
    name: string;
    inputs: {
        indexed: boolean;
        name: string;
        type: string;
    }[];
    outputs: {
        name: string;
        type: string;
    }[];
    payable: boolean;
    stateMutability: 'payable' | 'nonpayable' | 'view';
    type: 'event' | 'function' | 'constructor';
};
export declare class ExtensionContractBuilder implements ContractBuilder {
    private address;
    _abi: ABIMethod[];
    constructor(abi: string, address: string);
    call(methodName: string, ...args: any): Caller;
}
export declare class CPChainExtensionWallet implements Wallet {
    connected: boolean;
    chain?: Chain;
    constructor();
    contract(abi: string, contract_address: string): ContractBuilder;
    /**
     * Connnect with the extension wallet
     *
     * Errors:
     * - NotFoundExtensionWallet 插件钱包不存在
     * - WalletNotUnlocked 钱包未解锁
     * @returns {Promise<Account>}
     */
    connect(): Promise<Account>;
}
//# sourceMappingURL=extension_wallet.d.ts.map