"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CPChainExtensionWallet = exports.ExtensionContractBuilder = exports.ExtensionCaller = exports.ExtensionTxResult = exports.connectWallet = exports.extensionWalletExists = exports.submitTx = void 0;
const types_1 = require("./types");
const tx_helper_1 = require("./tx_helper");
const abi_1 = require("@ethersproject/abi");
function submitTx(payload) {
    return new Promise((resolve, reject) => {
        const listener = (event) => {
            if (event.data.type && (event.data.type === 'FROM_CPCHAIN')) {
                if (event.data.action === 'submit-tx') {
                    const data = event.data;
                    window.removeEventListener('message', listener);
                    resolve(data.payload);
                }
            }
        };
        window.addEventListener('message', listener);
        window.postMessage({
            type: 'TO_CPCHAIN',
            action: 'submit-tx',
            payload
        });
    });
}
exports.submitTx = submitTx;
function extensionWalletExists() {
    return new Promise((resolve, _reject) => {
        setTimeout(() => {
            resolve(!!document.getElementById("__cpchain_extension_wallet_checker"));
        }, 100);
    });
}
exports.extensionWalletExists = extensionWalletExists;
function connectWallet() {
    return new Promise((resolve, reject) => {
        const listener = (event) => {
            if (event.data.type && event.data.type === "FROM_CPCHAIN") {
                if (event.data.action === "connect") {
                    const data = event.data;
                    window.removeEventListener("message", listener);
                    if (data.payload.status === 'not_unlocked') {
                        reject(types_1.Errors.WalletNotUnlocked);
                    }
                    else {
                        resolve(data.payload);
                    }
                }
            }
        };
        window.addEventListener("message", listener);
        window.postMessage({ type: "TO_CPCHAIN", action: "connect" }, "*");
    });
}
exports.connectWallet = connectWallet;
class ExtensionTxResult {
    constructor(hash) {
        this.hash = hash;
    }
    wait() {
        throw new Error("Method not implemented.");
    }
}
exports.ExtensionTxResult = ExtensionTxResult;
class ExtensionCaller {
    constructor(method, to, data) {
        this.method = method;
        this.to = to;
        this.data = data;
        this._value = 0;
        this._gas = 25000;
    }
    view() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield fetch("https://civilian.cpchain.io", {
                method: 'POST',
                headers: {
                    'content-type': 'application/json'
                },
                body: JSON.stringify({
                    jsonrpc: '2.0',
                    method: 'eth_call',
                    id: '1',
                    params: [
                        {
                            to: this.to,
                            data: this.data
                        },
                        'latest'
                    ]
                })
            });
            const r = yield res.json();
            const outputs = this.method.outputs.map(i => i.type);
            const results = abi_1.defaultAbiCoder.decode(outputs, r.result);
            return results;
        });
    }
    gasLimit(gas) {
        this._gas = gas;
        return this;
    }
    value(value) {
        this._value = value;
        return this;
    }
    submit() {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = yield submitTx({
                to: this.to,
                data: this.data,
                value: this._value
            });
            if (tx.status === 'allowed' && tx.tx_hash) {
                return new ExtensionTxResult(tx.tx_hash);
            }
            throw new Error("Rejected");
        });
    }
}
exports.ExtensionCaller = ExtensionCaller;
class ExtensionContractBuilder {
    constructor(abi, address) {
        this.address = address;
        this._abi = JSON.parse(abi);
    }
    call(methodName, ...args) {
        for (const m of this._abi) {
            if (m.name === methodName) {
                // 将合约调用转化为 bytes
                const data = (0, tx_helper_1.encodeFunction)(m.name, {
                    types: m.inputs.map(i => i.type),
                    values: args
                });
                return new ExtensionCaller(m, this.address, data);
            }
        }
        throw new Error(`Method ${methodName} not found in your ABI`);
    }
}
exports.ExtensionContractBuilder = ExtensionContractBuilder;
class CPChainExtensionWallet {
    constructor() {
        this.connected = false;
    }
    contract(abi, contract_address) {
        // if (!this.connected || !this.chain) {
        //   throw new Error('Wallet not connected')
        // }
        return new ExtensionContractBuilder(abi, contract_address);
    }
    /**
     * Connnect with the extension wallet
     *
     * Errors:
     * - NotFoundExtensionWallet 插件钱包不存在
     * - WalletNotUnlocked 钱包未解锁
     * @returns {Promise<Account>}
     */
    connect() {
        this.connected = false;
        return extensionWalletExists()
            .then((exists) => {
            if (!exists) {
                throw new Error(types_1.Errors.NotFoundExtensionWallet);
            }
            return connectWallet();
        })
            .then((data) => {
            this.connected = true;
            this.chain = data.chain;
            return { address: data.address, chain: data.chain };
        }).catch(err => {
            throw new Error(err);
        });
    }
}
exports.CPChainExtensionWallet = CPChainExtensionWallet;
