
export interface Receipt {
  to: string;
  from: string;
  gasUsed: BigInt;
  contractAddress: string | null;
  transactionIndex: number;
  logsBloom: string;
  blockHash: string;
  transactionHash: string;
  blockNumber: number;
  confirmations: number;
  cumulativeGasUsed: BigInt;
  status: number;
  type: number;
  byzantium: boolean;
  logs: any[];
  events: any[];
}

export enum Errors {
  NotFoundExtensionWallet = "NotFoundExtensionWallet",
  WalletNotUnlocked = "WalletNotUnlocked",
}

export type Chain = {
  type: {
    name: string;
    logo?: string;
    path?: string;
  };
  name: string; // 链名称，非链类型名称
  isTestnet?: boolean; // 是否为测试链
  chainId: number;
  nodes: string[];
}

export type Account = {
  address: string;
  chain: Chain
};

export interface TxResult {
  hash: string;
  wait(): Promise<Receipt>;
}

export interface Caller {
  gasLimit(value: number): Caller;
  value(value: number): Caller;
  view(): Promise<unknown>; // 执行 view 方法
  submit(): Promise<TxResult>;
}

export interface ContractBuilder {
  call(methodName: string, ...args: any): Caller;
}

export interface Wallet {
  connected: boolean;
  connect(): Promise<Account>;
  contract(abi: string, contract_address: string): ContractBuilder;
}
